@@ -1,4 +1,4 @@
-/* $Id: ipc-daemon.c,v 1.21 2003/07/13 20:27:50 cwilson Exp $ */
+/* $Id: ipc-daemon.c,v 1.22 2003/10/17 04:59:00 cwilson Exp $ */
 
 /*
  *   IPC package for CygWin
@@ -260,6 +260,22 @@ static int create_map(const char *pcName,
 	return iRv;
 }
 
+void send_stop(void)
+{
+	if (hStopEvent) {
+		log_message(stderr, LOG_INFO, "sending hStopEvent");
+		SetEvent(hStopEvent);
+	}
+	else
+		log_message(stderr, LOG_INFO, "no hStopEvent to send");
+}
+
+void signal_handler(int sig)
+{
+	log_message(stderr, LOG_INFO, "received signal '%d'", sig);
+	send_stop();
+}
+
 int work_main(void)
 {
 	int iRv;
@@ -278,6 +294,10 @@ int work_main(void)
 	int LComptShm = 0 ;
 	int LRet ;
 
+	signal(SIGINT, signal_handler);
+	signal(SIGQUIT, signal_handler);
+	signal(SIGTERM, signal_handler);
+
 	do {	/* dummy loop */
 		iRv = 1;
 
@@ -664,16 +684,12 @@ service_handler(DWORD ctrl)
        cygwin_internal (CW_INIT_EXCEPTIONS, &except_list);
        switch (ctrl) {
        case SERVICE_CONTROL_STOP:
+       case SERVICE_CONTROL_SHUTDOWN:
                ss.dwCheckPoint = 1;
                ss.dwWaitHint = 3000L; // wait up to 3 seconds...
                ss.dwCurrentState = SERVICE_STOP_PENDING;
 
-               if (hStopEvent) {
-                 log_message(stderr, LOG_INFO, "sending hStopEvent");
-                 SetEvent(hStopEvent);
-               }
-               else
-                 log_message(stderr, LOG_INFO, "no hStopEvent to send");
+               send_stop();
 
                break;
        default:
@@ -701,7 +717,7 @@ service_main(DWORD argc, LPSTR *argv)
        }
        ss.dwServiceType = SERVICE_WIN32_SHARE_PROCESS;
        ss.dwCurrentState = SERVICE_START_PENDING;
-       ss.dwControlsAccepted = SERVICE_ACCEPT_STOP;
+       ss.dwControlsAccepted = SERVICE_ACCEPT_STOP | SERVICE_ACCEPT_SHUTDOWN;
        ss.dwWin32ExitCode = NO_ERROR;
        ss.dwCheckPoint = 1;
        ss.dwWaitHint = 3000L;
